@@ -5,7 +5,7 @@
 # ENV['RAILS_ENV'] ||= 'production'
 
 # Specifies gem version of Rails to use when vendor/rails is not present
-RAILS_GEM_VERSION = '2.3.5' unless defined? RAILS_GEM_VERSION
+RAILS_GEM_VERSION = '2.3.8' unless defined? RAILS_GEM_VERSION
 
 # Bootstrap the Rails environment, frameworks, and default configuration
 require File.join(File.dirname(__FILE__), 'boot')
@@ -27,8 +27,7 @@ Rails::Initializer.run do |config|
   # config.gem "hpricot", :version => '0.6', :source => "http://code.whytheluckystiff.net"
   # config.gem "sqlite3-ruby", :lib => "sqlite3"
   # config.gem "aws-s3", :lib => "aws/s3"
-  config.gem "locale_rails", :version => "2.0.5"
-  config.gem "exception_notification", :version => "1.0.20090728"
+  config.gem "exception_notification", :version => "2.3.3.0"
 
   # Only load the plugins named here, in the order given. By default, all plugins 
   # in vendor/plugins are loaded in alphabetical order.
@@ -60,7 +59,7 @@ Rails::Initializer.run do |config|
   # no regular words or you'll be exposed to dictionary attacks.
   secret_file = File.join(RAILS_ROOT, "config", "secret")
   unless File.exist?(secret_file)
-    require 'activesupport'
+    require 'active_support'
     secret = ActiveSupport::SecureRandom.hex(64)
     File.open(secret_file, "w") do |output|
       output.puts(secret)
@@ -68,7 +67,7 @@ Rails::Initializer.run do |config|
   end
   secret = File.read(secret_file).strip
   config.action_controller.session = {
-    :session_key => '_milter-manager-admin_session',
+    :key => '_milter-manager-admin_session',
     :secret      => secret,
   }
 
