@@ -13,12 +13,30 @@ run()
 
 svn_update()
 {
-    local repo="$1" dir="${2-`basename \"$1\"`}"
-    if [ $update != yes ]; then return; fi
+    local repository="$1"
+    local dir="${2-`basename $repository`}"
+    if [ $update != yes -a -d "$dir" ]; then
+	return
+    fi
     if test -d "$dir/.svn"; then
 	svn update "$dir"
     else
-	svn export --force "$repo" "$dir"
+	svn export --force "$repository" "$dir"
+    fi
+}
+
+git_update()
+{
+    local repository="$1"
+    local dir="${2-`basename $repository`}"
+    if [ $update != yes -a -d "$dir" ]; then
+	return
+    fi
+    if test -d "$dir/.git"; then
+	git pull --rebase "$dir"
+    else
+	rm -rf "$dir"
+	git clone "$repository" "$dir"
     fi
 }
 
@@ -38,8 +56,8 @@ run svn_update ${cutter_repository}/trunk/misc
 clear_code_repository=http://www.clear-code.com/repos/svn
 run svn_update ${clear_code_repository}/tdiary html/blog/clear-code
 
-test_unit_repository=http://test-unit.rubyforge.org/svn
-run svn_update ${test_unit_repository}/trunk binding/ruby/test-unit
+test_unit_repository=https://github.com/test-unit/test-unit.git
+run git_update ${test_unit_repository} binding/ruby/test-unit
 
 run ${ACLOCAL:-aclocal} $ACLOCAL_OPTIONS
 run ${LIBTOOLIZE:-libtoolize} --copy --force
