@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby" -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,5 @@
 void
 Init_milter_utils (void)
 {
-    VALUE rb_mMilterUtils;
-
-    rb_mMilterUtils = rb_define_module_under(rb_mMilter, "Utils");
+    rb_define_module_under(rb_mMilter, "Utils");
 }
