@@ -22,14 +22,18 @@ require 'milter/client/composite-session'
 
 module Milter
   class Client
-    def status_on_error
-      @status_on_error ||= :accept
+    def fallback_status
+      @fallback_status ||= :accept
     end
 
-    def status_on_error=(status)
-      @status_on_error = status
+    def fallback_status=(status)
+      @fallback_status = status
     end
 
+    # just for backward compatibility.
+    alias_method :status_on_error, :fallback_status
+    alias_method :status_on_error=, :fallback_status=
+
     def register(session_class, *new_arguments)
       signal_connect("connection-established") do |_client, context|
         setup_session(context, session_class, new_arguments)
@@ -38,25 +42,33 @@ module Milter
 
     def on_error
       signal_connect("error") do |_client, error|
-        yield(_client, error)
+        Milter::Callback.guard do
+          yield(_client, error)
+        end
       end
     end
 
     def on_maintain
       signal_connect("maintain") do |_client|
-        yield(_client)
+        Milter::Callback.guard do
+          yield(_client)
+        end
       end
     end
 
     def on_event_loop_created
-      signal_connect("event-loop-created") do |_client|
-        yield(_client)
+      signal_connect("event-loop-created") do |_client, event_loop|
+        Milter::Callback.guard do
+          yield(_client, event_loop)
+        end
       end
     end
 
     def reload
       reload_callbacks.each do |callback|
-        callback.call(self)
+        Milter::Callback.guard do
+          callback.call(self)
+        end
       end
     end
 
@@ -83,7 +95,7 @@ module Milter
             end
           rescue Exception
             Milter::Logger.error($!)
-            session_context.status = status_on_error
+            session_context.status = fallback_status
           end
           status = session_context.status
           session_context.clear
