@@ -69,7 +69,7 @@ module Milter
         attr_accessor :suspend_time_on_unacceptable, :max_connections
         attr_accessor :max_file_descriptors, :event_loop_backend
         attr_accessor :n_workers, :packet_buffer_size
-        attr_accessor :status_on_error
+        attr_accessor :fallback_status
         attr_writer :daemon, :handle_signal, :run_gc_on_maintain
         def initialize(base_configuration)
           @base_configuration = base_configuration
@@ -102,6 +102,7 @@ module Milter
             Milter::Client::DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE
           @max_connections = Milter::Client::DEFAULT_MAX_CONNECTIONS
           @max_file_descriptors = 0
+          @fallback_status = "accept"
           @event_loop_backend = Milter::Client::EVENT_LOOP_BACKEND_GLIB.nick
           @n_workers = 0
           @packet_buffer_size = 0
@@ -112,10 +113,10 @@ module Milter
         end
 
         def setup(client)
-          client.status_on_error = @status_on_error
           client.connection_spec = @connection_spec
           client.unix_socket_group = @unix_socket_group
           client.unix_socket_mode = @unix_socket_mode if @unix_socket_mode
+          client.fallback_status = @fallback_status
           client.event_loop_backend = resolved_event_loop_backend
           client.default_packet_buffer_size = @packet_buffer_size
           client.pid_file = @pid_file
@@ -230,7 +231,7 @@ module Milter
 
       class DatabaseConfiguration
         attr_accessor :type, :name, :host, :port, :path
-        attr_accessor :user, :password
+        attr_accessor :user, :password, :extra_options
         def initialize(base_configuration)
           @base_configuration = base_configuration
           @loaded_model_files = []
@@ -246,6 +247,7 @@ module Milter
           @path = nil
           @user = nil
           @password = nil
+          @extra_options = {}
           unless @loaded_model_files.empty?
             $LOADED_FEATURES.reject! do |required_path|
               @loaded_model_files.include?(required_path)
@@ -291,7 +293,7 @@ module Milter
             :path => path,
             :user => user,
             :password => password,
-          }
+          }.merge(extra_options)
         end
 
         def update_location(key, reset, deep_level)
@@ -310,12 +312,13 @@ module Milter
         def active_record_options
           case @type
           when "mysql", "mysql2"
-            mysql_options
+            options = mysql_options
           when "sqlite3"
-            sqlite3_options
+            options = sqlite3_options
           else
-            default_options
+            options = default_options
           end
+          options.merge(extra_options)
         end
 
         def mysql_options
@@ -347,11 +350,15 @@ module Milter
         end
 
         def default_options
-          options = to_hash
-          options[:adapter] = options.delete(:type)
-          options[:database] = options.delete(:name)
-          options[:username] = options.delete(:user)
-          options
+          {
+            :adapter => type,
+            :database => name,
+            :host => host,
+            :port => port,
+            :path => path,
+            :username => user,
+            :password => password,
+          }
         end
 
         def expand_path(path)
@@ -389,15 +396,6 @@ module Milter
         end
       end
 
-      class << self
-        def guard(fallback_value=nil)
-          yield
-        rescue Exception => error
-          Milter::Logger.error(error)
-          fallback_value
-        end
-      end
-
       attr_reader :milter, :security, :log, :database
       def initialize(configuration)
         @configuration = configuration
@@ -450,8 +448,11 @@ module Milter
       end
 
       class MilterConfigurationLoader
+        attr_reader :available_fallback_statuses
         def initialize(configuration)
           @configuration = configuration
+          @available_fallback_statuses = ["accept", "reject",
+                                          "temporary-failure", "discard"]
         end
 
         def connection_spec
@@ -558,14 +559,22 @@ module Milter
           @configuration.max_file_descriptors = n_descriptors
         end
 
-        def connection_check_interval
-          @configuration.connection_check_interval
+        def fallback_status
+          @configuration.fallback_status
         end
 
-        def connection_check_interval=(interval)
-          update_location("connection_check_interval", interval.nil?)
-          interval ||= 0
-          @configuration.connection_check_interval = interval
+        def fallback_status=(status)
+          available_values = @available_fallback_statuses
+          normalized_status = status
+          unless status.nil?
+            normalized_status = status.to_s.downcase.gsub(/_/, '-')
+            unless available_values.include?(normalized_status)
+              raise InvalidValue.new(full_key("fallback_status"),
+                                     available_values, status)
+            end
+          end
+          update_location("fallback_status", status.nil?)
+          @configuration.fallback_status = normalized_status
         end
 
         def event_loop_backend
@@ -599,7 +608,7 @@ module Milter
 
         def maintained(hook=Proc.new)
           guarded_hook = Proc.new do |configuration|
-            ConfigurationLoader.guard do
+            Milter::Callback.guard do
               hook.call
             end
           end
@@ -608,7 +617,7 @@ module Milter
 
         def event_loop_created(hook=Proc.new)
           guarded_hook = Proc.new do |configuration, loop|
-            ConfigurationLoader.guard do
+            Milter::Callback.guard do
               hook.call(loop)
             end
           end
@@ -617,8 +626,11 @@ module Milter
 
         private
         def update_location(key, reset, deep_level=2)
-          full_key = "milter.#{key}"
-          @configuration.update_location(full_key, reset, deep_level)
+          @configuration.update_location(full_key(key), reset, deep_level)
+        end
+
+        def full_key(key)
+          "milter.#{key}"
         end
       end
 
@@ -759,6 +771,16 @@ module Milter
           @configuration.password = password
         end
 
+        def extra_options
+          @configuration.extra_options
+        end
+
+        def extra_options=(options)
+          options ||= {}
+          update_location("extra_options", options.empty?)
+          @configuration.extra_options = options
+        end
+
         def setup
           missing_values = @configuration.missing_values
           unless missing_values.empty?
