@@ -202,6 +202,7 @@ module Milter
         dump_item("manager.packet_buffer_size", c.default_packet_buffer_size)
         dump_item("manager.connection_check_interval",
                   c.connection_check_interval.inspect)
+        dump_item("manager.chunk_size", c.chunk_size)
         @result << "\n"
       end
 
@@ -335,7 +336,7 @@ module Milter
         @@current_configuration = nil
         def load(configuration, file)
           @@current_configuration = configuration
-          Client::ConfigurationLoader.guard do
+          Milter::Callback.guard do
             new(configuration).load_configuration(file)
           end
           GC.start
@@ -343,7 +344,7 @@ module Milter
 
         def load_custom(configuration, file)
           @@current_configuration = configuration
-          Client::ConfigurationLoader.guard do
+          Milter::Callback.guard do
             new(configuration).load_custom_configuration(file)
           end
           GC.start
@@ -896,12 +897,12 @@ module Milter
             @configuration.max_file_descriptors = n_descriptors
           end
 
-          def connection_check_interval
-            @configuration.connection_check_interval
+          def fallback_status
+            @raw_configuration.fallback_status
           end
 
-          def connection_check_interval=(interval)
-            @configuration.connection_check_interval = interval
+          def fallback_status=(status)
+            @configuration.fallback_status = status
           end
 
           def event_loop_backend
@@ -937,8 +938,7 @@ module Milter
           end
 
           def update_location(key, reset, deep_level=2)
-            full_key = "manager.#{key}"
-            @configuration.update_location(full_key, reset, deep_level)
+            @configuration.update_location(key, reset, deep_level)
           end
         end
 
@@ -957,15 +957,6 @@ module Milter
           @raw_configuration.custom_configuration_directory = directory
         end
 
-        def fallback_status
-          @raw_configuration.fallback_status
-        end
-
-        def fallback_status=(status)
-          update_location("fallback_status", status.nil?)
-          @raw_configuration.fallback_status = status
-        end
-
         def fallback_status_at_disconnect
           @raw_configuration.fallback_status_at_disconnect
         end
@@ -975,16 +966,36 @@ module Milter
           @raw_configuration.fallback_status_at_disconnect = status
         end
 
+        def chunk_size
+          @raw_configuration.chunk_size
+        end
+
+        def chunk_size=(size)
+          update_location("chunk_size", size.nil?)
+          size ||= Milter::CHUNK_SIZE
+          @raw_configuration.chunk_size = size
+        end
+
+        def connection_check_interval
+          @raw_configuration.connection_check_interval
+        end
+
+        def connection_check_interval=(interval)
+          update_location("connection_check_interval", interval.nil?)
+          interval ||= 0
+          @raw_configuration.connection_check_interval = interval
+        end
+
         def netstat_connection_checker
           @raw_configuration.netstat_connection_checker
         end
 
         def define_connection_checker(name, &block)
           old_id = @connection_checker_ids[name]
-          @configuration.signal_handler_disconnect(old_id) if old_id
-          id = @configuration.signal_connect('connected') do |_config, leader|
+          @raw_configuration.signal_handler_disconnect(old_id) if old_id
+          id = @raw_configuration.signal_connect('connected') do |config, leader|
             leader.signal_connect('connection-check') do |_leader|
-              Client::ConfigurationLoader.guard(true) do
+              Milter::Callback.guard(true) do
                 block.call(ConnectionCheckContext.new(leader))
               end
             end
@@ -1016,7 +1027,11 @@ module Milter
 
         private
         def update_location(key, reset, deep_level=2)
-          @configuration.update_location(key, reset, deep_level + 1)
+          super(key, reset, deep_level + 2)
+        end
+
+        def full_key(key)
+          "manager.#{key}"
         end
 
         class MemoryReporter
@@ -1262,7 +1277,7 @@ module Milter
               child.signal_connect("stop-on-connect") do |_child, host, address|
                 child_context = ChildContext.new(child, children, context)
                 @connect_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context, host, address)
                   end
                 end
@@ -1273,7 +1288,7 @@ module Milter
               child.signal_connect("stop-on-helo") do |_child, fqdn|
                 child_context = ChildContext.new(child, children, context)
                 @helo_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context, fqdn)
                   end
                 end
@@ -1284,7 +1299,7 @@ module Milter
               child.signal_connect("stop-on-envelope-from") do |_child, from|
                 child_context = ChildContext.new(child, children, context)
                 @envelope_from_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context, from)
                   end
                 end
@@ -1295,7 +1310,7 @@ module Milter
               child.signal_connect("stop-on-envelope-recipient") do |_child, recipient|
                 child_context = ChildContext.new(child, children, context)
                 @envelope_recipient_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context, recipient)
                   end
                 end
@@ -1306,7 +1321,7 @@ module Milter
               child.signal_connect("stop-on-data") do |_child|
                 child_context = ChildContext.new(child, children, context)
                 @data_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context)
                   end
                 end
@@ -1317,7 +1332,7 @@ module Milter
               child.signal_connect("stop-on-header") do |_child, name, value|
                 child_context = ChildContext.new(child, children, context)
                 @header_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context, name, value)
                   end
                 end
@@ -1328,7 +1343,7 @@ module Milter
               child.signal_connect("stop-on-end-of-header") do |_child|
                 child_context = ChildContext.new(child, children, context)
                 @end_of_header_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context)
                   end
                 end
@@ -1339,7 +1354,7 @@ module Milter
               child.signal_connect("stop-on-body") do |_child, chunk|
                 child_context = ChildContext.new(child, children, context)
                 @body_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context, name, chunk)
                   end
                 end
@@ -1350,7 +1365,7 @@ module Milter
               child.signal_connect("stop-on-end-of-message") do |_child, chunk|
                 child_context = ChildContext.new(child, children, context)
                 @end_of_message_stoppers.any? do |stopper|
-                  Client::ConfigurationLoader.guard(false) do
+                  Milter::Callback.guard(false) do
                     stopper.call(child_context)
                   end
                 end
