@@ -53,7 +53,7 @@ module Milter
     def guess_milter_test_server_path
       path = ENV["MILTER_TEST_SERVER"]
       if defined?(::PKGConfig)
-        path ||= PKGConfig.variable("milter-test-server",
+        path ||= PKGConfig.variable("milter-server",
                                     "milter_test_server")
       end
       path || "milter-test-server"
