@@ -9,26 +9,20 @@ module Test
     class TestTestCase < TestCase
       self.test_order = :random
       def test_creation
-        tc = Class.new(TestCase) do
+        test_case = Class.new(TestCase) do
           def test_with_arguments(arg1, arg2)
           end
         end
-      
-        caught = true
-        catch(:invalid_test) do
-          tc.new(:test_with_arguments)
-          caught = false
-        end
-        check("Should have caught an invalid test when there are arguments", caught)
-        
-        caught = true
-        catch(:invalid_test) do
-          tc.new(:non_existent_test)
-          caught = false
-        end
-        check("Should have caught an invalid test when the method does not exist", caught)
+
+        test = test_case.new(:test_with_arguments)
+        check("Should have caught an invalid test when there are arguments",
+              !test.valid?)
+
+        test = test_case.new(:non_existent_test)
+        check("Should have caught an invalid test when the method does not exist",
+              !test.valid?)
       end
-      
+
       def setup
         @tc_failure_error = Class.new(TestCase) do
           def test_failure
@@ -75,7 +69,11 @@ module Test
         test_case.run(result) { |*arguments| progress << arguments }
         check("The failure should have triggered the listener", called)
         check("The failure should have set passed?", !test_case.return_passed?)
-        check("The progress block should have been updated correctly", [[TestCase::STARTED, test_case.name], [TestCase::FINISHED, test_case.name]] == progress)
+        check("The progress block should have been updated correctly",
+              [[TestCase::STARTED, test_case.name],
+               [TestCase::STARTED_OBJECT, test_case],
+               [TestCase::FINISHED, test_case.name],
+               [TestCase::FINISHED_OBJECT, test_case]] == progress)
       end
 
       def test_add_failure_nested
@@ -441,17 +439,14 @@ module Test
           end
         end
 
-        assert_nothing_thrown do
-          test_case.new("test_nothing")
-        end
+        test = test_case.new("test_nothing")
+        assert_predicate(test, :valid?)
 
-        assert_nothing_thrown do
-          sub_test_case.new("test_fail")
-        end
+        test = sub_test_case.new("test_fail")
+        assert_predicate(test, :valid?)
 
-        assert_throw(:invalid_test) do
-          sub_test_case.new("test_nothing")
-        end
+        test = sub_test_case.new("test_nothing")
+        assert_not_predicate(test, :valid?)
       end
 
       def test_mixin_test_should_not_be_ignored
@@ -511,9 +506,9 @@ module Test
 
         test_case.test_order = :defined
 
-        assert_equal(["test_declarative_style_test_definition",
-                      "test_include_parenthesis",
-                      "test_1_2_3"],
+        assert_equal(["declarative style test definition",
+                      "include parenthesis",
+                      "1 + 2 = 3"],
                      test_case.suite.tests.collect {|test| test.method_name})
 
         assert_equal(["declarative style test definition",
@@ -522,6 +517,19 @@ module Test
                      test_case.suite.tests.collect {|test| test.description})
       end
 
+      def test_test_mark
+        test_case = Class.new(Test::Unit::TestCase) do
+          test
+          def my_test_method
+          end
+        end
+
+        test_case.test_order = :defined
+
+        assert_equal(["my_test_method"],
+                     test_case.suite.tests.collect {|test| test.method_name})
+      end
+
       def test_redefine_method
         test_case = Class.new(Test::Unit::TestCase) do
           def test_name
@@ -542,6 +550,18 @@ module Test
                      result.summary)
       end
 
+      def test_data_driven_test
+        test_case = Class.new(TestCase) do
+          def test_with_data(data)
+          end
+        end
+
+        test = test_case.new("test_with_data")
+        assert_not_predicate(test, :valid?)
+        test.assign_test_data("label1", :test_data1)
+        assert_predicate(test, :valid?)
+      end
+
       private
       def check(message, passed)
         add_assertion
