@@ -232,6 +232,19 @@ class TestConfigurationLoader < Test::Unit::TestCase
     assert_equal(0, @configuration.default_packet_buffer_size)
   end
 
+  def test_manager_chunk_size
+    assert_equal(65535, @configuration.chunk_size)
+    @loader.manager.chunk_size = 4096
+    assert_equal(4096, @configuration.chunk_size)
+    @loader.manager.chunk_size = nil
+    assert_equal(65535, @configuration.chunk_size)
+  end
+
+  def test_manager_chunk_size_over
+    @loader.manager.chunk_size = 65536
+    assert_equal(65535, @configuration.chunk_size)
+  end
+
   def test_database_type
     assert_equal(nil, @configuration.database.type)
     @loader.database.type = "mysql"
@@ -505,4 +518,40 @@ EOX
                    [condition.name, condition.description]
                  end)
   end
+
+  class TestCallback < TestConfigurationLoader
+    include MilterEventLoopTestUtils
+
+    def test_maintained
+      n_called = {
+        :before => 0,
+        :after => 0,
+      }
+      @loader.manager.maintained do
+        n_called[:before] += 1
+        raise "failed"
+        n_called[:after] += 1
+      end
+      assert_nothing_raised do
+        @configuration.maintained
+      end
+      assert_equal({:before => 1, :after => 0}, n_called)
+    end
+
+    def test_event_loop_created
+      n_called = {
+        :before => 0,
+        :after => 0,
+      }
+      @loader.manager.event_loop_created do |loop|
+        n_called[:before] += 1
+        raise "failed"
+        n_called[:after] += 1
+      end
+      assert_nothing_raised do
+        @configuration.event_loop_created(create_event_loop)
+      end
+      assert_equal({:before => 1, :after => 0}, n_called)
+    end
+  end
 end
