@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 #
-# Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,25 +19,15 @@ require 'pathname'
 
 base = Pathname.new(__FILE__).dirname.expand_path
 top = (base + "..").expand_path
-test_unit_dir = top + "test-unit" + "lib"
 
+test_unit_dir = top + "test-unit" + "lib"
 $LOAD_PATH.unshift(test_unit_dir.to_s)
 require 'test/unit'
 
-if ENV["NO_MAKE"] != "yes"
-  make = "make"
-  make = "gmake" if system("which gmake > /dev/null")
-  system("#{make} -C #{top.to_s.dump} > /dev/null") or exit(1)
-end
-
-$LOAD_PATH.unshift((top + "src" + "toolkit" + ".libs").to_s)
-$LOAD_PATH.unshift((top + "src" + "manager" + ".libs").to_s)
-$LOAD_PATH.unshift((top + "lib").to_s)
+require 'milter/manager'
 
 $LOAD_PATH.unshift(base.to_s)
 require 'milter-test-utils'
 
-require 'milter/manager'
-
 ARGV.unshift("--priority-mode")
 exit Test::Unit::AutoRunner.run(true, base)
