@@ -928,26 +928,40 @@ change_unix_socket_group (MilterClient *client, struct sockaddr_un *address_un)
     errno = 0;
     group = getgrnam(socket_group);
     if (!group) {
+        GError *error;
         if (errno == 0) {
-            milter_error(
-                "[client][error][unix] "
-                "failed to find group entry for UNIX socket group: <%s>: <%s>",
-                address_un->sun_path, socket_group);
+            error = g_error_new(MILTER_CLIENT_ERROR,
+                                MILTER_CLIENT_ERROR_UNIX_SOCKET,
+                                "failed to find group entry "
+                                "for UNIX socket group: <%s>: <%s>",
+                                address_un->sun_path, socket_group);
+            milter_error("[client][error][unix] %s", error->message);
         } else {
-            milter_error(
-                "[client][error][unix] "
-                "failed to get group entry for UNIX socket group: "
-                "<%s>: <%s>: %s",
-                address_un->sun_path, socket_group, g_strerror(errno));
+            error = g_error_new(MILTER_CLIENT_ERROR,
+                                MILTER_CLIENT_ERROR_UNIX_SOCKET,
+                                "failed to get group entry "
+                                "for UNIX socket group: <%s>: <%s>: %s",
+                                address_un->sun_path,
+                                socket_group,
+                                g_strerror(errno));
+            milter_error("[client][error][unix] %s", error->message);
         }
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(client), error);
+        g_error_free(error);
         return;
     }
 
     if (chown(address_un->sun_path, -1, group->gr_gid) == -1) {
-        milter_error(
-            "[client][error][unix] "
-            "failed to change UNIX socket group: <%s>: <%s>: %s",
-            address_un->sun_path, socket_group, g_strerror(errno));
+        GError *error;
+        error = g_error_new(MILTER_CLIENT_ERROR,
+                            MILTER_CLIENT_ERROR_UNIX_SOCKET,
+                            "failed to change UNIX socket group: <%s>: <%s>: %s",
+                            address_un->sun_path,
+                            socket_group,
+                            g_strerror(errno));
+        milter_error("[client][error][unix] %s", error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(client), error);
+        g_error_free(error);
     } else {
         milter_info("[client][socket][unix][group][change] <%s>(<%d>): <%s>",
                     group->gr_name, group->gr_gid,
@@ -1502,13 +1516,11 @@ multi_thread_process_client_channel (MilterClient *client, GIOChannel *channel,
 static gboolean
 multi_thread_accept_connection (MilterClient *client, gint server_fd)
 {
-    MilterClientPrivate *priv;
     gboolean accepted;
     GIOChannel *client_channel;
     MilterGenericSocketAddress address;
     socklen_t address_size;
 
-    priv = MILTER_CLIENT_GET_PRIVATE(client);
     accepted = accept_connection(client, server_fd, &client_channel,
                                  &address, &address_size);
     if (accepted) {
@@ -1880,12 +1892,9 @@ accept_error_watch_func (GIOChannel *channel, GIOCondition condition,
                          gpointer user_data)
 {
     MilterClient *client = user_data;
-    MilterClientPrivate *priv;
     gchar *message;
     GError *error = NULL;
 
-    priv = MILTER_CLIENT_GET_PRIVATE(client);
-
     message = milter_utils_inspect_io_condition_error(condition);
     error = g_error_new(MILTER_CLIENT_ERROR,
                         MILTER_CLIENT_ERROR_IO_ERROR,
@@ -2326,11 +2335,8 @@ worker_accept_watch_func (GIOChannel *channel, GIOCondition condition,
     gint server_fd, client_fd;
     MilterGenericSocketAddress address;
     socklen_t address_size;
-    MilterClientPrivate *priv;
     gboolean keep_callback = TRUE;
 
-    priv = MILTER_CLIENT_GET_PRIVATE(client);
-
     server_fd = g_io_channel_unix_get_fd(channel);
     client_fd = accept_connection_fd(client, server_fd, &address, &address_size);
     if (client_fd == -1) {
