@@ -35,10 +35,10 @@ def login(agent, user_name, password=nil)
 
   begin
     page = agent.submit(login_form, login_form.buttons.first)
-  rescue WWW::Mechanize::ResponseCodeError
+  rescue Mechanize::ResponseCodeError
   end
   raise "login failed" unless /Log Out|Log&nbsp;out/ =~ page.body
-  page = agent.get("https://sourceforge.net/my/")
+  page = agent.get("https://sourceforge.net/users/#{user_name}")
   page
 end
 
@@ -56,8 +56,8 @@ def upload_files(sf_user_name, project_id, package_name, release_name,
   dist_dir = File.join(dist_top_dir, package_name, release_name)
   FileUtils.mkdir_p(dist_dir)
   FileUtils.cp(files, dist_dir)
-  File.open(File.join(dist_dir, "#{release_name}-release.txt"), "w") do |note|
-    note.print(latest_release_changes(news))
+  File.open(File.join(dist_dir, "README.txt"), "w") do |readme|
+    readme.print(latest_release_changes(news))
   end
   Dir.chdir(dist_top_dir) do
     host = "#{sf_user_name},#{project_id}@frs.sourceforge.net"
@@ -67,11 +67,15 @@ def upload_files(sf_user_name, project_id, package_name, release_name,
   end
 end
 
-def go_development_page(agent, project_page)
+def go_development_page(agent, project_page, project_id)
   development_page_link = project_page.links.find do |link|
-    /\bprojects\b/ =~ link.href and /\ADevelop\z/ =~ link.text
+    /\bprojects\b/ =~ link.href and /\bDevelop\b/ =~ link.text
+  end
+  if development_page_link.nil?
+    agent.get("https://sourceforge.net/projects/#{project_id}/develop")
+  else
+    agent.click(development_page_link)
   end
-  agent.click(development_page_link)
 end
 
 def extract_sections(file)
@@ -128,14 +132,14 @@ end
 
 def main(sf_user_name, project_id, project_name,
          package_name, release_name, readme, news, files)
-  agent = WWW::Mechanize.new do |_agent|
+  agent = Mechanize.new do |_agent|
     # _agent.log = Logger.new(STDOUT)
   end
   password = read_password("SF.net password for [#{sf_user_name}]: ")
   my_page = login(agent, sf_user_name, password)
 
   project_page = go_project_page(agent, my_page, project_name)
-  development_page = go_development_page(agent, project_page)
+  development_page = go_development_page(agent, project_page, project_id)
   upload_files(sf_user_name, project_id, package_name,
                release_name, news, files)
 
