@@ -58,6 +58,8 @@ void test_default_packet_buffer_size (void);
 void test_prefix (void);
 void test_use_syslog (void);
 void test_syslog_facility (void);
+void test_chunk_size (void);
+void test_chunk_size_over (void);
 void test_egg (void);
 void test_find_egg (void);
 void test_remove_egg (void);
@@ -607,6 +609,27 @@ test_syslog_facility (void)
         milter_manager_configuration_get_syslog_facility(config));
 }
 
+void
+test_chunk_size (void)
+{
+    cut_assert_equal_uint(
+        MILTER_CHUNK_SIZE,
+        milter_manager_configuration_get_chunk_size(config));
+    milter_manager_configuration_set_chunk_size(config, 29);
+    cut_assert_equal_uint(
+        29,
+        milter_manager_configuration_get_chunk_size(config));
+}
+
+void
+test_chunk_size_over (void)
+{
+    milter_manager_configuration_set_chunk_size(config, MILTER_CHUNK_SIZE + 1);
+    cut_assert_equal_uint(
+        MILTER_CHUNK_SIZE,
+        milter_manager_configuration_get_chunk_size(config));
+}
+
 static void
 milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
 {
@@ -698,6 +721,10 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
         NULL,
         milter_manager_configuration_get_syslog_facility(config));
 
+    cut_assert_equal_uint(
+        MILTER_CHUNK_SIZE,
+        milter_manager_configuration_get_chunk_size(config));
+
     if (expected_children)
         g_object_unref(expected_children);
     expected_children = milter_manager_children_new(config, loop);
@@ -920,6 +947,7 @@ test_clear (void)
     test_default_packet_buffer_size();
     test_use_syslog();
     test_syslog_facility();
+    test_chunk_size();
 
     handler_id = g_signal_connect(config, "connected",
                                   G_CALLBACK(cb_connected), NULL);
