@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,8 @@
 
 void test_establish_connection (void);
 void test_establish_connection_failure (void);
+void test_negotiate (void);
+void test_connect (void);
 void test_helo (void);
 void test_envelope_from (void);
 void test_envelope_recipient (void);
@@ -47,6 +49,10 @@ void test_state (void);
 void test_last_state (void);
 void test_macro (void);
 void test_macros_hash_table (void);
+void data_has_accepted_recipient (void);
+void test_has_accepted_recipient (gconstpointer data);
+
+static MilterEventLoop *loop;
 
 static MilterServerContext *context;
 static MilterTestClient *client;
@@ -54,6 +60,8 @@ static MilterTestClient *client;
 static MilterReplyEncoder *encoder;
 static MilterDecoder *decoder;
 
+static MilterOption *option;
+
 static MilterStatus reply_status;
 static gboolean command_received;
 static gboolean reply_received;
@@ -64,7 +72,6 @@ static MilterMessageResult *message_result;
 
 static GError *actual_error;
 static GError *expected_error;
-static gchar *packet;
 
 static void
 write_data (const gchar *data, gsize data_size)
@@ -76,10 +83,9 @@ write_data (const gchar *data, gsize data_size)
 static void
 send_continue (void)
 {
+    const gchar *packet;
     gsize packet_size;
 
-    if (packet)
-        g_free(packet);
     milter_reply_encoder_encode_continue(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
 }
@@ -87,10 +93,9 @@ send_continue (void)
 static void
 send_temporary_failure (void)
 {
+    const gchar *packet;
     gsize packet_size;
 
-    if (packet)
-        g_free(packet);
     milter_reply_encoder_encode_temporary_failure(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
 }
@@ -98,10 +103,9 @@ send_temporary_failure (void)
 static void
 send_reject (void)
 {
+    const gchar *packet;
     gsize packet_size;
 
-    if (packet)
-        g_free(packet);
     milter_reply_encoder_encode_reject(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
 }
@@ -109,10 +113,9 @@ send_reject (void)
 static void
 send_discard (void)
 {
+    const gchar *packet;
     gsize packet_size;
 
-    if (packet)
-        g_free(packet);
     milter_reply_encoder_encode_discard(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
 }
@@ -120,14 +123,29 @@ send_discard (void)
 static void
 send_accept (void)
 {
+    const gchar *packet;
     gsize packet_size;
 
-    if (packet)
-        g_free(packet);
     milter_reply_encoder_encode_accept(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
 }
 
+static void
+cb_negotiate_received (MilterDecoder *decoder, gpointer user_data)
+{
+    const gchar *packet;
+    gsize packet_size;
+    MilterMacrosRequests *macros_requests;
+
+    command_received = TRUE;
+
+    macros_requests = milter_macros_requests_new();
+    milter_reply_encoder_encode_negotiate(encoder, &packet, &packet_size,
+                                          option, macros_requests);
+    g_object_unref(macros_requests);
+    write_data(packet, packet_size);
+}
+
 static void
 cb_command_received (MilterDecoder *decoder, gpointer user_data)
 {
@@ -188,6 +206,7 @@ setup_server_context_signals (MilterServerContext *context)
 #define CONNECT(name)                                                   \
     g_signal_connect(context, #name, G_CALLBACK(cb_reply_received), NULL)
 
+    CONNECT(negotiate_reply);
     CONNECT(continue);
     CONNECT(temporary_failure);
     CONNECT(reject);
@@ -211,10 +230,12 @@ setup_server_context_signals (MilterServerContext *context)
 static void
 setup_command_signals (MilterDecoder *decoder)
 {
+    g_signal_connect(decoder, "negotiate", G_CALLBACK(cb_negotiate_received),
+                     NULL);
+
 #define CONNECT(name)                                                   \
     g_signal_connect(decoder, #name, G_CALLBACK(cb_command_received), NULL)
 
-    CONNECT(negotiate);
     CONNECT(define_macro);
     CONNECT(connect);
     CONNECT(helo);
@@ -232,11 +253,13 @@ setup_command_signals (MilterDecoder *decoder)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
-    MilterOption *option;
+    loop = milter_test_event_loop_new();
 
     context = milter_server_context_new();
+    milter_server_context_set_name(context, "test-server-context");
+    milter_agent_set_event_loop(MILTER_AGENT(context), loop);
     setup_server_context_signals(context);
 
     option = milter_option_new(6,
@@ -251,7 +274,6 @@ setup (void)
                                MILTER_ACTION_SET_SYMBOL_LIST,
                                MILTER_STEP_SKIP);
     milter_server_context_set_option(context, option);
-    g_object_unref(option);
 
     client = NULL;
 
@@ -259,7 +281,6 @@ setup (void)
     decoder = milter_command_decoder_new();
     setup_command_signals(decoder);
 
-    packet = NULL;
     actual_error = NULL;
     expected_error = NULL;
 
@@ -271,10 +292,10 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
-    if (packet)
-        g_free(packet);
+    if (option)
+        g_object_unref(option);
 
     if (client)
         g_object_unref(client);
@@ -293,12 +314,15 @@ teardown (void)
         g_error_free(actual_error);
     if (expected_error)
         g_error_free(expected_error);
+
+    if (loop)
+        g_object_unref(loop);
 }
 
 static void
 setup_test_client (const gchar *spec)
 {
-    cut_trace(client = milter_test_client_new(spec, decoder));
+    cut_trace(client = milter_test_client_new(spec, decoder, loop));
 }
 
 static gboolean
@@ -316,12 +340,13 @@ wait_ready (void)
     gboolean timeout_waiting = TRUE;
     guint timeout_waiting_id;
 
-    timeout_waiting_id = g_timeout_add_seconds(1, cb_timeout_waiting,
-                                               &timeout_waiting);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 1,
+                                                       cb_timeout_waiting,
+                                                       &timeout_waiting);
     while (timeout_waiting && !ready_received) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
     cut_assert_true(timeout_waiting, cut_message("timeout"));
 }
 
@@ -331,12 +356,13 @@ wait_error (void)
     gboolean timeout_waiting = TRUE;
     guint timeout_waiting_id;
 
-    timeout_waiting_id = g_timeout_add_seconds(2, cb_timeout_waiting,
-                                               &timeout_waiting);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 2,
+                                                       cb_timeout_waiting,
+                                                       &timeout_waiting);
     while (timeout_waiting && !actual_error) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
     cut_assert_true(timeout_waiting, cut_message("timeout"));
 }
 
@@ -391,7 +417,7 @@ wait_for_receiving_command (void)
     command_received = FALSE;
 
     while (!command_received) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
 }
 
@@ -399,16 +425,66 @@ static void
 wait_for_receiving_reply (void)
 {
     while (!reply_received) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
 }
 
+void
+test_negotiate (void)
+{
+    cut_trace(test_establish_connection());
+
+    milter_server_context_negotiate(context, option);
+    cut_assert_false(milter_server_context_is_negotiated(context));
+
+    wait_for_receiving_command();
+    cut_assert_false(milter_server_context_is_negotiated(context));
+
+    wait_for_receiving_reply();
+    cut_assert_true(milter_server_context_is_negotiated(context));
+
+    gcut_assert_equal_enum(MILTER_TYPE_STATUS,
+                           MILTER_STATUS_NOT_CHANGE,
+                           milter_server_context_get_status(context));
+}
+
+void
+test_connect (void)
+{
+    struct sockaddr_in address;
+    const gchar host_name[] = "mx.example.com";
+    const gchar ip_address[] = "192.168.123.123";
+    guint16 port;
+
+    cut_trace(test_negotiate());
+
+    port = g_htons(50443);
+    address.sin_family = AF_INET;
+    address.sin_port = port;
+    inet_pton(AF_INET, ip_address, &(address.sin_addr));
+
+    milter_server_context_connect(context, host_name,
+                                  (struct sockaddr *)&address,
+                                  sizeof(address));
+    cut_assert_true(milter_server_context_is_processing(context));
+
+    wait_for_receiving_command();
+    cut_assert_true(milter_server_context_is_processing(context));
+
+    wait_for_receiving_reply();
+    cut_assert_false(milter_server_context_is_processing(context));
+
+    gcut_assert_equal_enum(MILTER_TYPE_STATUS,
+                           MILTER_STATUS_NOT_CHANGE,
+                           milter_server_context_get_status(context));
+}
+
 void
 test_helo (void)
 {
     const gchar fqdn[] = "delian";
 
-    cut_trace(test_establish_connection());
+    cut_trace(test_connect());
 
     milter_server_context_helo(context, fqdn);
     cut_assert_true(milter_server_context_is_processing(context));
@@ -722,6 +798,82 @@ test_macros_hash_table (void)
         milter_protocol_agent_get_available_macros(agent));
 }
 
+void
+data_has_accepted_recipient (void)
+{
+#define ADD(label, expected, recipients)                         \
+    gcut_add_datum(label " - [" #expected "] <" recipients ">",  \
+                   "expected", G_TYPE_BOOLEAN, expected,         \
+                   "recipients", G_TYPE_STRING, recipients,      \
+                   NULL)
+
+    ADD("no message result", TRUE, "");
+    ADD("all accepted", TRUE,
+        "alice@example.com,"
+        "bob@example.com");
+    ADD("one accepted - no unaccepted", TRUE,
+        "alice@example.com");
+    ADD("one accepted - one rejected", TRUE,
+        "alice@example.com,"
+        "bob@example.com:reject");
+    ADD("one accepted - one temporary failed", TRUE,
+        "alice@example.com,"
+        "bob@example.com:temporary-failure");
+    ADD("no accepted - one rejected", FALSE,
+        "alice@example.com:reject");
+    ADD("no accepted - one temporary failed", FALSE,
+        "alice@example.com:temporary-failure");
+    ADD("all unaccepted", FALSE,
+        "alice@example.com:temporary-failure,"
+        "bob@example.com:reject");
+
+#undef ADD
+}
+
+void
+test_has_accepted_recipient (gconstpointer data)
+{
+    const gchar *recipients_text;
+    const gchar **recipients;
+
+    cut_trace(test_envelope_from());
+
+    recipients_text = gcut_data_get_string(data, "recipients");
+    recipients = cut_take_string_array(g_strsplit(recipients_text, ",", 0));
+    for (; *recipients; recipients++) {
+        gchar **recipient_and_action;
+        const gchar *recipient, *action = NULL;
+
+        recipient_and_action = g_strsplit(*recipients, ":", 2);
+        recipient = cut_take_strdup(recipient_and_action[0]);
+        if (recipient_and_action[1])
+            action = cut_take_strdup(recipient_and_action[1]);
+        g_strfreev(recipient_and_action);
+
+        if (cut_equal_string(action, "reject")) {
+            reply_status = MILTER_STATUS_REJECT;
+        } else if (cut_equal_string(action, "temporary-failure")) {
+            reply_status = MILTER_STATUS_TEMPORARY_FAILURE;
+        } else {
+            reply_status = MILTER_STATUS_CONTINUE;
+        }
+        milter_server_context_envelope_recipient(context, recipient);
+        cut_assert_true(milter_server_context_is_processing(context));
+
+        wait_for_receiving_command();
+        cut_assert_true(milter_server_context_is_processing(context));
+
+        wait_for_receiving_reply();
+        cut_assert_false(milter_server_context_is_processing(context));
+    }
+
+    if (gcut_data_get_boolean(data, "expected")) {
+        cut_assert_true(milter_server_context_has_accepted_recipient(context));
+    } else {
+        cut_assert_false(milter_server_context_has_accepted_recipient(context));
+    }
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
